/**
ATR for Stops
Shows ATR above and below the current prices

This could be readily used by an EA via iCustom().

This calculates the ATR based upon the formula, not by pulling the values
from the ATR example indicator nor iATR().

One guy mentioned that he uses ATR(20) on D1 to set his SL.
He suggests that prices stay within the ATR(20) 80% of the time.

Tommor uses a stop-loss at 2•ATR(20) from entry to have a
volatility-related stop distance.

*/

#property copyright		"2018 Anthony J. Garot"
#property link			"http://www.garot.com/trading/"
#property version		"2.0"
#property icon			"\\Images\\ATR_for_Stops-icon.ico"
#property description	"ATR for Stops"
#property description	" "
#property description	"MQL5 Profile: https://www.mql5.com/en/users/tonegarot"
#property description	"My Website: http://www.garot.com/trading/"
#property strict


//--- indicator settings
//#property indicator_separate_window
#property indicator_chart_window
#property indicator_buffers				6
#property indicator_plots				4

#property indicator_type1				DRAW_LINE
#property indicator_style1				STYLE_DOT
#property indicator_color1				DodgerBlue
#property indicator_label1				"ATR R1"

#property indicator_type2				DRAW_LINE
#property indicator_style2				STYLE_DOT
#property indicator_color2				DodgerBlue
#property indicator_label2				"ATR S1"

#property indicator_type3				DRAW_LINE
#property indicator_style3				STYLE_DOT
#property indicator_color3				SeaGreen
#property indicator_label3				"ATR R2"

#property indicator_type4				DRAW_LINE
#property indicator_style4				STYLE_DOT
#property indicator_color4				SeaGreen
#property indicator_label4				"ATR S2"

#include <incIndicators.mqh>

enum INP_APPLY_TO
{
	AT_HIGH_LOW,		// High/Low
	AT_CLOSE_PRICE,		// Close price
	AT_OPEN_PRICE		// Open price
};

//---- constants
const int 			MAIN_WINDOW = 0;
const int 			SUB_WINDOW = 1;
const int 			chartID = 0;

//---- globals
int 				g_periodATR;			// This allows me to set a different value if InpAtrPeriod is an incorrect value.
int					g_linesAppliedToBar;	// which bar to show the HLine levels. Typically 1, which is last filled.
string				g_shortName = "A4S";
int					DEBUG = 0;				// The greater the number, the more verbose. 0=off;1=verbose;2=spray;3=diarrhea

//---- input parameters
input int					InpAtrPeriod=20;				// ATR period

input INP_APPLY_TO			InpS1R1ApplyTo = AT_HIGH_LOW;	// Apply To for S1/R1
// 1.0 = ATR Envelopes
// < 1.0 might be used to set price for stop buy/sell orders
input double				InpS1R1Multiplier=0.35;			// Multiplier for S1/R1;

input INP_APPLY_TO			InpS2R2ApplyTo = AT_CLOSE_PRICE;// Apply To for S2/R2
/* Tommor uses 2x for SL's */
input double				InpS2R2Multiplier=2.;			// Multiplier for S2/R2

//---- indicator buffers/arrays
double	ExtATRBufferR1[];
double	ExtATRBufferS1[];
double	ExtATRBufferR2[];
double	ExtATRBufferS2[];
double	ATRBuffer[];			// internal
double	TRBuffer[];				// internal


int OnInit()
{
	if ( DEBUG )
	{
		PrintFormat("%s(): max bars [%d]", __FUNCTION__, TerminalInfoInteger(TERMINAL_MAXBARS) );
		PrintFormat("%s(): MQL_PROGRAM_TYPE [%d]",	__FUNCTION__, MQLInfoInteger(MQL_PROGRAM_TYPE));
		PrintFormat("%s(): MQL_PROGRAM_NAME [%s]",	__FUNCTION__, MQLInfoString(MQL_PROGRAM_NAME));
		PrintFormat("%s(): MQL5_PROGRAM_PATH [%s]",	__FUNCTION__, MQLInfoString(MQL_PROGRAM_PATH));
	}

	//--- check for input value
	g_periodATR=InpAtrPeriod;
	if(g_periodATR < 1)
	{
		g_periodATR=14;
		Print("Incorrect input parameter InpAtrPeriod = %d. Indicator will use value %d for calculations.",InpAtrPeriod,g_periodATR);
	}

	//--- indicator buffers mapping
	SetIndexBuffer(0,ExtATRBufferR1,	INDICATOR_DATA);
	SetIndexBuffer(1,ExtATRBufferS1,	INDICATOR_DATA);
	SetIndexBuffer(2,ExtATRBufferR2,	INDICATOR_DATA);
	SetIndexBuffer(3,ExtATRBufferS2,	INDICATOR_DATA);
	SetIndexBuffer(4,ATRBuffer,			INDICATOR_CALCULATIONS);
	SetIndexBuffer(5,TRBuffer,			INDICATOR_CALCULATIONS);

	IndicatorSetInteger(INDICATOR_DIGITS,_Digits);

	ChartSetInteger(chartID,CHART_SHIFT,true);
	ChartSetInteger(chartID,CHART_AUTOSCROLL,true);		// the mode of automatic shift to the right border of the chart.

	// This sets graphical objects to the foreground, i.e. the grid and
	// bars will be hidden behind the graphical objects.
	ChartSetInteger(chartID, CHART_FOREGROUND, false);

	//--- sets first bar from what index will be drawn
	PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,InpAtrPeriod);

	Comment("");

	//--- name for DataWindow and indicator subwindow label
	IndicatorSetString(INDICATOR_SHORTNAME,g_shortName);

	// initialization done
	return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason)
{
	if ( DEBUG ) PrintFormat("%s(): reason [%d]", __FUNCTION__, reason);
}

int OnCalculate(const int rates_total,
				const int prev_calculated,
				const datetime &time[],
				const double &open[],
				const double &high[],
				const double &low[],
				const double &close[],
				const long &tick_volume[],
				const long &volume[],
				const int &spread[])
{
	//--- check for bars count
	if(rates_total<=g_periodATR)
	{
		return(0); // not enough bars for calculation
	}

	// correct position, when it's first iteration
	int pos = prev_calculated - 1;
	if ( pos <= 0 )
	{
		pos = 1;
	}

	// Do all functions you cannot do in OnInit() here.
	if ( prev_calculated == 0 )
	{
		ArrayInitialize(TRBuffer,0.);
		ArrayInitialize(ATRBuffer,0.);

		//--- filling out the array of True Range values for each period
		for(int j=1;j<rates_total && !IsStopped();j++)
		{
			TRBuffer[j]=MathMax(high[j],close[j-1])-MathMin(low[j],close[j-1]);
		}

		//--- first AtrPeriod values of the indicator are not calculated
		double firstValue=0.0;
		for(int j=1;j<=g_periodATR;j++)
		{
			ATRBuffer[j]=0.0;
			firstValue+=TRBuffer[j];
		}

		//--- calculating the first value of the indicator
		firstValue/=g_periodATR;
		ATRBuffer[g_periodATR]=firstValue;
		pos=g_periodATR+1;
	}

	//--- the main loop of calculations
	for(int i=pos;i<rates_total && !IsStopped();i++)
	{
		TRBuffer[i]=MathMax(high[i],close[i-1])-MathMin(low[i],close[i-1]);
		ATRBuffer[i]=ATRBuffer[i-1]+(TRBuffer[i]-TRBuffer[i-g_periodATR])/g_periodATR;

		double distanceS1R1 = InpS1R1Multiplier * ATRBuffer[i];		// Multiplier for S1/R1; typically <= 1.0
		double distanceS2R2 = InpS2R2Multiplier * ATRBuffer[i];		// Multiplier for S2/R2; typically 2.0

		if ( DEBUG && MathMod(i,200) == 0 )
		{
			PrintFormat("i [%d] rates_total [%d] ATRBuffer [%f] distanceS2R2 [%f]", i, rates_total, ATRBuffer[i], distanceS2R2);
		}

		if ( InpS1R1ApplyTo == AT_HIGH_LOW )
		{
			// Add to the current hi/lo
			ExtATRBufferR1[i] = high[i] + distanceS1R1;
			ExtATRBufferS1[i] = low[i] - distanceS1R1;
		}
		else if ( InpS1R1ApplyTo == AT_CLOSE_PRICE )		// Probably the one I will use most
		{
			ExtATRBufferR1[i] = close[i] + distanceS1R1;	// Add ATR to close
			ExtATRBufferS1[i] = close[i] - distanceS1R1;
		}
		else if ( InpS1R1ApplyTo == AT_OPEN_PRICE )
		{
			ExtATRBufferR1[i] = open[i] + distanceS1R1;
			ExtATRBufferS1[i] = open[i] - distanceS1R1;
		}

		if ( InpS2R2ApplyTo == AT_HIGH_LOW )
		{
			// Add to the current hi/lo
			ExtATRBufferR2[i] = high[i] + distanceS2R2;
			ExtATRBufferS2[i] = low[i] - distanceS2R2;
		}
		else if ( InpS2R2ApplyTo == AT_CLOSE_PRICE )		// Probably the one I will use most
		{
			ExtATRBufferR2[i] = close[i] + distanceS2R2;	// Add N•ATR to close
			ExtATRBufferS2[i] = close[i] - distanceS2R2;
		}
		else if ( InpS2R2ApplyTo == AT_OPEN_PRICE )
		{
			ExtATRBufferR2[i] = open[i] + distanceS2R2;
			ExtATRBufferS2[i] = open[i] - distanceS2R2;
		}
	}

	//--- return value for next call
	return(rates_total);
}

void OnChartEvent(const int id,         // Event identifier
                  const long& lparam,   // Event parameter of long type
                  const double& dparam, // Event parameter of double type
                  const string& sparam  // Event parameter of string type, max 159 chars
                  )
{
	// keystrokes; KEYSTROKES
	// In native MQL there is no KEYUP event
	if( id == CHARTEVENT_KEYDOWN )
	{

		if ( DEBUG ) PrintFormat("CHARTEVENT_CHART_CHANGE id [%d] lparam [%d] dparam [%f] sparam [%s]", id, lparam, dparam, sparam);

		switch(int(lparam))
		{
		case KEY_STAR: // keypad *
			{
				// I could also do regular star by checking TERMINAL_KEYSTATE_SHIFT and key 8.

				string filename = StringFormat(	"%s - %s - ATR for Stops.png",
												TimeToString(TimeCurrent(), TIME_DATE),
												Symbol() );
				WrapChartScreenShot(filename, MAIN_WINDOW);
			}
			break;

		case KEY_C:
			{
				Print("Executing ChartRedraw()");
				ChartRedraw();
			}
			break;

		case KEY_Q:		// 'q'
			{
				int keystate = TerminalInfoInteger(TERMINAL_KEYSTATE_CONTROL);
				if ( keystate == -127 || keystate == -128 )
				{
					// This is quicker than right-clicking -> Remove
					Print("CTRL-Q hit. Removing Indicator");
					ChartIndicatorDelete(chartID,MAIN_WINDOW,g_shortName);
				}
			}
			break;
		}
	}
}
